import abc
from .. import typing as tp
from abc import ABC, abstractmethod

class Operation(ABC, metaclass=abc.ABCMeta):
    def __call__(self, **params) -> tp.Callable: ...
    @abstractmethod
    def operation(self, obj, **params) -> tp.Any: ...

class Property(ABC, metaclass=abc.ABCMeta):
    @abstractmethod
    def get(self) -> tp.Any: ...
    @property
    def value(self) -> tp.Any: ...
    def transform(self, operation: Operation, **kwargs) -> Property: ...

class Constant(Property):
    def __init__(self, value) -> None: ...
    def get(self) -> tp.Any: ...

class Source(Property):
    def __init__(self, fn: tp.Callable, *args, **kwargs) -> None: ...
    def get(self) -> tp.Any: ...

class Convert(Source):
    def __init__(self, p: LikeProperty, fn: tp.Callable, *args, **kwargs) -> None: ...
    def get(self) -> tp.Any: ...

class Iter(Property):
    def __init__(self, seq: tp.Iterable) -> None: ...
    def get(self) -> tp.Any: ...

class RandomUniform(Property):
    def __init__(self, low: float = ..., high: float = ...) -> None: ...
    def get(self): ...

class RandomChoice(Property):
    def __init__(self, items) -> None: ...
    def get(self): ...

class Transformer:
    def __init__(self, *operations: Operation) -> None: ...
    def __call__(self, instance: Property) -> Property: ...
